#include "__anynet2g_api.h"
#include "Click_AnyNet_2G_types.h"

/* -----------------------------------------------------------------------------
    ISR
------------------------------------------------------------------------------*/

void TICK_ISR() iv IVT_INT_TIM2
{
    TIM2_SR.UIF = 0;
    anynet2g_tick();
}

/*
 * UART Interrupt
 */
void RX_ISR() iv IVT_INT_USART3 ics ICS_AUTO
{
    if( RXNE_USART3_SR_bit )
    {
        char tmp = USART3_DR;
        anynet2g_putc( tmp );
    }
}

/*----------------------------------------------------------------------------*/

static const uint8_t _LOG_LINE = 0;
static const uint8_t _LOG_BYTE = 1;
static const uint8_t _LOG_TEXT = 2;

static void mikrobus_logWrite( char *input, uint8_t logt )
{
    if (logt == _LOG_LINE)
    {
        UART1_Write_Text( input );
        UART1_Write( 13 );
        UART1_Write( 10 );
    }
    if (logt == _LOG_BYTE)
    {
        UART1_Write( *input );
    }
    if (logt == _LOG_TEXT)
    {
        UART1_Write_Text( input );
    }
}

static void print_header( char *x ) 
{
    mikrobus_logWrite( "-----------------------------------------", _LOG_LINE ); 
    mikrobus_logWrite( x, _LOG_LINE );                                         
    mikrobus_logWrite( "-----------------------------------------", _LOG_LINE );
    mikrobus_logWrite( "", _LOG_LINE );
}

static void print_footer() 
{
    mikrobus_logWrite( "", _LOG_LINE );                                          
    mikrobus_logWrite( "-----------------------------------------", _LOG_LINE );
    mikrobus_logWrite( "", _LOG_LINE ); 
}

/** 
 * API INIT
 *
 * - FLAGS
 * - TIMER
 *
 * USE MIKROE TIMER CALCULATOR TO GENERATE 1ms TIMER CONFIGURATION
 */
void api_anynet2g_init()
{
// USE MIKROE TIMER CALCULATOR TO GENERATE 1ms TIMER CONFIGURATION
    RCC_APB1ENR.TIM2EN = 1;
    TIM2_CR1.CEN = 0;
    TIM2_PSC = 1;
    TIM2_ARR = 35999;
    TIM2_DIER.UIE = 1;
    TIM2_CR1.CEN = 1;;
    RXNEIE_USART3_CR1_bit = 1;
    NVIC_IntEnable( IVT_INT_TIM2 );
    NVIC_IntEnable( IVT_INT_USART3 );
    EnableInterrupts();
}

void api_anynet2g_firmwareID()
{
    anynet2g_cmdSingle( "AT+AWSVER" );
}

void api_anynet2g_queryICCID()
{
    anynet2g_cmdSingle( "AT+QCCID" );
}

void api_anynet2g_openTopicAWS( uint8_t cmdType, char* awsIdx, char* awsTopic )
{
    if (_ANYNET2G_TEST == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSPUBOPEN=?" );
    }
    else if (_ANYNET2G_GET == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSPUBOPEN?" );
    }
    else if (_ANYNET2G_SET == cmdType)
    {
        char tmp[255];

        strcpy( tmp, "AT+AWSPUBOPEN=" );
        strcat( tmp, awsIdx );
        strcat( tmp, ",\"" );
        strcat( tmp, awsTopic );
        strcat( tmp, "\"" );
        anynet2g_cmdSingle( tmp );
    }
    else
    {
        mikrobus_logWrite( "< < < UNSUPPORTED COMMAND > > >", _LOG_LINE );
        mikrobus_logWrite( "", _LOG_LINE );
    }
}

void api_anynet2g_publishTopicAWS( uint8_t cmdType, char* awsIdx, char* msg, char* qos )
{
    if (_ANYNET2G_TEST == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSPUBLISH=?" );
    }
    else if (_ANYNET2G_GET == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSPUBLISH?" );
    }
    if (_ANYNET2G_SET == cmdType)
    {
        uint16_t len_d;

        char len[10];
        char tmp[255];

        len_d = strlen( msg );
        WordToStr( len_d, len );
        Ltrim( len );

        strcpy( tmp, "AT+AWSPUBLISH=" );
        strcat( tmp, awsIdx );
        strcat( tmp, "," );
        strcat( tmp, len );

        if (0 != *qos )
        {
            strcat( tmp, "," );
            strcat( tmp, qos );
        }

        anynet2g_cmdDouble( tmp, msg );
    }
    else
    {
        mikrobus_logWrite( "< < < UNSUPPORTED COMMAND > > >", _LOG_LINE );
        mikrobus_logWrite( "", _LOG_LINE );
    }

}

void api_anynet2g_openSubscribeAWS( uint8_t cmdType, char* awsIdx, char* awsTopic )
{
    char tmp[255];
    
    if (_ANYNET2G_TEST == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSSUBOPEN=?" );
    }
    else if (_ANYNET2G_GET == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSSUBOPEN?" );
    }
    else if (_ANYNET2G_SET == cmdType)
    {
        strcpy( tmp, "AT+AWSSUBOPEN=" );
        strcat( tmp, awsIdx );
        strcat( tmp, "," );
        strcat( tmp, "\"" );
        strcat( tmp, awsTopic );
        strcat( tmp, "\"" );

        anynet2g_cmdSingle( tmp );
    }
    else
    {
        mikrobus_logWrite( "< < < UNSUPPORTED COMMAND > > >", _LOG_LINE );
        mikrobus_logWrite( "", _LOG_LINE );
    }
}

void api_anynet2g_closeTopicAWS( uint8_t cmdType, char* awsIdx )
{
    if (_ANYNET2G_TEST == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSPUBCLOSE=?" );
    }
    else if (_ANYNET2G_GET == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSPUBCLOSE?" );
    }
    else if (_ANYNET2G_SET == cmdType)
    {
        char tmp[255];

        strcpy( tmp, "AT+AWSPUBCLOSE=" );
        strcat( tmp, awsIdx );
        anynet2g_cmdSingle( tmp );
    }
    else
    {
        mikrobus_logWrite( "< < < UNSUPPORTED COMMAND > > >", _LOG_LINE );
        mikrobus_logWrite( "", _LOG_LINE );
    }
}

void api_anynet2g_checkStateAWS( uint8_t cmdType )
{
    if (_ANYNET2G_TEST == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSSTATE=?" );
    }
    else if (_ANYNET2G_GET == cmdType)
    {
        anynet2g_cmdSingle( "AT+AWSSTATE?" );
    }
    else
    {
        mikrobus_logWrite( "< < < UNSUPPORTED COMMAND > > >", _LOG_LINE );
        mikrobus_logWrite( "", _LOG_LINE );
    }
}

/* -----------------------------------------------------------------------------
    HANDLERS
------------------------------------------------------------------------------*/

void api_anynet2g_default_handler( char *rsp, uint8_t *evArgs )
{
    mikrobus_logWrite( rsp, _LOG_LINE );
}

void api_anynet2g_firmwareID_handler( char *rsp, uint8_t *evArgs )
{
    char *tmp = strchr( rsp, 10 );
    
    print_header( "Firmware ID" );
    
    mikrobus_logWrite( tmp, _LOG_LINE );
    
    print_footer();
}

void api_anynet2g_queryICCID_handler( char *rsp, uint8_t *evArgs )
{
    char *tmp = strchr( rsp, 10 );

    print_header( "QCC ID" );

    mikrobus_logWrite( tmp, _LOG_LINE );

    print_footer();
}

void api_anynet2g_openTopicAWS_handler( char *rsp, uint8_t *evArgs )
{
    print_header( "Open AWS Publish Topic" );

    if (_ANYNET2G_SET == evArgs[_ANYNET2G_EVARG_END_T])
    {
        char *tmp;
        tmp = strchr( rsp, ':' ) + 1;

        mikrobus_logWrite( "Topic ", _LOG_TEXT );
        mikrobus_logWrite( tmp, _LOG_BYTE );

        tmp = strchr( tmp, ',' ) + 1;

        if (*tmp == '0')
        {
            mikrobus_logWrite( " successfully opened.", _LOG_LINE );
        }
        else
        {
            if (*(tmp + 1) == '1')
            {
                mikrobus_logWrite( " error.", _LOG_LINE );
            }
            else if (*(tmp + 1) == '2')
            {
                mikrobus_logWrite( " socket already in use.", _LOG_LINE );
            }
        }
    }
    else
    {
        mikrobus_logWrite( "UNKNOWN RESPONSE", _LOG_LINE );
    }

    print_footer();
}

void api_anynet2g_openSubscribeAWS_handler( char *rsp, uint8_t *evArgs )
{
    print_header( "Open AWS Subscribe Topic" );

    if (_ANYNET2G_SET == evArgs[_ANYNET2G_EVARG_END_T])
    {
        char *tmp;
        char *end;
        char txt[255];

        tmp = strchr( rsp, ':' ) + 1;

        mikrobus_logWrite( "Topic ", _LOG_TEXT );
        mikrobus_logWrite( tmp, _LOG_BYTE );
        
        tmp = strchr( tmp, ',' ) + 1;

        if (*tmp == '0')
        {
            mikrobus_logWrite( " successfully subscribed.", _LOG_LINE );
        }
        else
        {
            if (*(tmp + 1) == '1')
            {
                mikrobus_logWrite( " error.", _LOG_LINE );
            }
            else if (*(tmp + 1) == '2')
            {
                mikrobus_logWrite( " socket not subscribed.", _LOG_LINE );
            }
        }
    }
    else
    {
        mikrobus_logWrite( "UNKNOWN RESPONSE", _LOG_LINE );
    }

    print_footer();
}

void api_anynet2g_closeTopicAWS_handler( char *rsp, uint8_t *evArgs )
{
    print_header( "Close AWS Publish Topic" );

    if (_ANYNET2G_SET == evArgs[_ANYNET2G_EVARG_END_T])
    {
        char *tmp;
        char *end;
        char txt[255];

        tmp = strchr( rsp, ':' ) + 1;

        mikrobus_logWrite( "Topic ", _LOG_TEXT );
        mikrobus_logWrite( tmp, _LOG_BYTE );

        tmp = strchr( tmp, ',' ) + 1;

        if (*tmp == '0')
        {
            mikrobus_logWrite( " successfully closed.", _LOG_LINE );
        }
        else
        {
            if (*(tmp + 1) == '1')
            {
                mikrobus_logWrite( " error.", _LOG_LINE );
            }
            else if (*(tmp + 1) == '2')
            {
                mikrobus_logWrite( " socket not open.", _LOG_LINE );
            }
        }
    }
    else
    {
        mikrobus_logWrite( "UNKNOWN RESPONSE", _LOG_LINE );
    }

    print_footer();
}

void api_anynet2g_checkStateAWS_handler( char *rsp, uint8_t *evArgs )
{
    print_header( "Check AWS State" );

    if (_ANYNET2G_GET == evArgs[_ANYNET2G_EVARG_END_T])
    {
        char *tmp;
        tmp = strchr( rsp, ':' ) + 2;

        mikrobus_logWrite( "Current AWS State: ", _LOG_TEXT );

        switch (*tmp)
        {
            case '1' :
            {
                mikrobus_logWrite( "IDLE", _LOG_LINE );
                break;
            }
            case '2' :
            {
                mikrobus_logWrite( "WAITING KEYS", _LOG_LINE );
                break;
            }
            case '3' :
            {
                mikrobus_logWrite( "CONNECTING TO NETWORK...", _LOG_LINE );
                break;
            }
            case '4' :
            {
                mikrobus_logWrite( "ESTABLISHING SSL", _LOG_LINE );
                break;
            }
            case '5' :
            {
                mikrobus_logWrite( "SSL CONNECTED", _LOG_LINE );
                break;
            }
            case '6' :
            {
                mikrobus_logWrite( "CONNECTING MQTT...", _LOG_LINE );
                break;
            }
            case '7' :
            {
                mikrobus_logWrite( "READY - MQTT CONNECTED", _LOG_LINE );
                break;
            }
            case '8' :
            {
                mikrobus_logWrite( "READY - SUBSCRIBED!", _LOG_LINE );
                break;
            }
            case '9' :
            {
                mikrobus_logWrite( "ERROR", _LOG_LINE );
                break;
            }
            default :
            {
                mikrobus_logWrite( "UNKNOWN RESPONSE", _LOG_LINE );
                break;
            }
        }
    }

    print_footer();
}

void api_anynet2g_newAWS_handler( char *rsp, uint8_t *evArgs )
{
     print_header( "AWS Received" );
     
     mikrobus_logWrite( rsp, _LOG_LINE );
     
     print_footer();
}